Package: xfun
Type: Package
Title: Supporting Functions for Packages Maintained by 'Yihui Xie'
Version: 0.52
Authors@R: c(
  person("Yihui", "Xie", role = c("aut", "cre", "cph"), email = "xie@yihui.name", comment = c(ORCID = "0000-0003-0645-5666", URL = "https://yihui.org")),
  person("Wush", "Wu", role = "ctb"),
  person("Daijiang", "Li", role = "ctb"),
  person("Xianying", "Tan", role = "ctb"),
  person("Salim", "Brüggemann", role = "ctb", email = "salim-b@pm.me", comment = c(ORCID = "0000-0002-5329-5987")),
  person("Christophe", "Dervieux", role = "ctb"),
  person()
  )
Description: Miscellaneous functions commonly used in other packages maintained by 'Yihui Xie'.
Depends: R (>= 3.2.0)
Imports: grDevices, stats, tools
Suggests: testit, parallel, codetools, methods, rstudioapi, tinytex (>=
        0.30), mime, litedown (>= 0.4), commonmark, knitr (>= 1.50),
        remotes, pak, curl, xml2, jsonlite, magick, yaml, qs
License: MIT + file LICENSE
URL: https://github.com/yihui/xfun
BugReports: https://github.com/yihui/xfun/issues
Encoding: UTF-8
RoxygenNote: 7.3.2
VignetteBuilder: litedown
NeedsCompilation: yes
Packaged: 2025-04-02 19:53:53 UTC; runner
Author: Yihui Xie [aut, cre, cph] (<https://orcid.org/0000-0003-0645-5666>,
    https://yihui.org),
  Wush Wu [ctb],
  Daijiang Li [ctb],
  Xianying Tan [ctb],
  Salim Brüggemann [ctb] (<https://orcid.org/0000-0002-5329-5987>),
  Christophe Dervieux [ctb]
Maintainer: Yihui Xie <xie@yihui.name>
Repository: CRAN
Date/Publication: 2025-04-02 20:40:02 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4; 2026-07-17 18:00:41 UTC; unix
