Package: xfun
Type: Package
Title: Supporting Functions for Packages Maintained by 'Yihui Xie'
Version: 0.58
Authors@R: c(
  person("Yihui", "Xie", role = c("aut", "cre", "cph"), email = "xie@yihui.name", comment = c(ORCID = "0000-0003-0645-5666", URL = "https://yihui.org")),
  person("Wush", "Wu", role = "ctb"),
  person("Daijiang", "Li", role = "ctb"),
  person("Xianying", "Tan", role = "ctb"),
  person("Salim", "Brüggemann", role = "ctb", email = "salim-b@pm.me", comment = c(ORCID = "0000-0002-5329-5987")),
  person("Christophe", "Dervieux", role = "ctb"),
  person()
  )
Description: Miscellaneous functions commonly used in other packages maintained by 'Yihui Xie'.
Depends: R (>= 3.2.0)
Imports: grDevices, stats, tools
Suggests: testit, parallel, codetools, methods, rstudioapi, tinytex (>=
        0.30), mime, litedown (>= 0.6), commonmark, knitr (>= 1.50),
        remotes, pak, curl, xml2, jsonlite, magick, yaml, data.table,
        qs2
License: MIT + file LICENSE
URL: https://github.com/yihui/xfun
BugReports: https://github.com/yihui/xfun/issues
Encoding: UTF-8
VignetteBuilder: litedown
Config/roxygen2/version: 8.0.0
NeedsCompilation: yes
Packaged: 2026-05-31 16:49:17 UTC; yihui
Author: Yihui Xie [aut, cre, cph] (ORCID:
    <https://orcid.org/0000-0003-0645-5666>, URL: https://yihui.org),
  Wush Wu [ctb],
  Daijiang Li [ctb],
  Xianying Tan [ctb],
  Salim Brüggemann [ctb] (ORCID: <https://orcid.org/0000-0002-5329-5987>),
  Christophe Dervieux [ctb]
Maintainer: Yihui Xie <xie@yihui.name>
Repository: CRAN
Date/Publication: 2026-06-01 05:10:03 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-06-01 05:37:19 UTC; unix
Archs: xfun.so.dSYM
