Package: xfun
Type: Package
Title: Supporting Functions for Packages Maintained by 'Yihui Xie'
Version: 0.36
Authors@R: c(
  person("Yihui", "Xie", role = c("aut", "cre", "cph"), email = "xie@yihui.name", comment = c(ORCID = "0000-0003-0645-5666")),
  person("Wush", "Wu", role = "ctb"),
  person("Daijiang", "Li", role = "ctb"),
  person("Xianying", "Tan", role = "ctb"),
  person("Salim", "Brüggemann", role = "ctb", email = "salim-b@pm.me", comment = c(ORCID = "0000-0002-5329-5987")),
  person("Christophe", "Dervieux", role = "ctb"),
  person()
  )
Description: Miscellaneous functions commonly used in other packages maintained by 'Yihui Xie'.
Imports: stats, tools
Suggests: testit, parallel, codetools, rstudioapi, tinytex (>= 0.30),
        mime, markdown, knitr, htmltools, remotes, pak, rhub, renv,
        curl, jsonlite, magick, rmarkdown
License: MIT + file LICENSE
URL: https://github.com/yihui/xfun
BugReports: https://github.com/yihui/xfun/issues
Encoding: UTF-8
RoxygenNote: 7.2.3
VignetteBuilder: knitr
NeedsCompilation: yes
Packaged: 2022-12-21 15:05:22 UTC; yihui
Author: Yihui Xie [aut, cre, cph] (<https://orcid.org/0000-0003-0645-5666>),
  Wush Wu [ctb],
  Daijiang Li [ctb],
  Xianying Tan [ctb],
  Salim Brüggemann [ctb] (<https://orcid.org/0000-0002-5329-5987>),
  Christophe Dervieux [ctb]
Maintainer: Yihui Xie <xie@yihui.name>
Repository: CRAN
Date/Publication: 2022-12-21 15:40:02 UTC
