% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paths.R
\name{del_empty_dir}
\alias{del_empty_dir}
\title{Delete an empty directory}
\usage{
del_empty_dir(dir)
}
\arguments{
\item{dir}{Path to a directory. If \code{NULL} or the directory does not
exist, no action will be performed.}
}
\description{
Use \code{list.file()} to check if there are any files or subdirectories
under a directory. If not, delete this empty directory.
}
