% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/packages.R
\name{install_dir}
\alias{install_dir}
\title{Install a source package from a directory}
\usage{
install_dir(pkg = ".", build = TRUE, build_opts = NULL, install_opts = NULL)
}
\arguments{
\item{pkg}{The package source directory.}

\item{build}{Whether to build a tarball from the source directory. If
\code{FALSE}, run \command{R CMD INSTALL} on the directory directly (note
that vignettes will not be automatically built).}

\item{build_opts}{The options for \command{R CMD build}.}

\item{install_opts}{The options for \command{R CMD INSTALL}.}
}
\value{
Invisible status from \command{R CMD INSTALL}.
}
\description{
Run \command{R CMD build} to build a tarball from a source directory, and run
\command{R CMD INSTALL} to install it.
}
