% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paths.R
\name{dir_exists}
\alias{dir_exists}
\alias{file_exists}
\title{Test the existence of files and directories}
\usage{
dir_exists(x)

file_exists(x)
}
\arguments{
\item{x}{A vector of paths.}
}
\value{
A logical vector.
}
\description{
These are wrapper functions of [\verb{utils::file_test()]} to test the
existence of directories and files. Note that \code{file_exists()} only tests
files but not directories, which is the main difference between
\code{\link[=file.exists]{file.exists()}} in base R. If you use are using the R version
3.2.0 or above, \code{dir_exists()} is the same as \code{\link[=dir.exists]{dir.exists()}}
in base R.
}
