% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/packages.R
\name{base_pkgs}
\alias{base_pkgs}
\title{Get base R package names}
\usage{
base_pkgs()
}
\value{
A character vector of base R package names.
}
\description{
Return base R package names.
}
\examples{
xfun::base_pkgs()
}
