% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/knitr.R
\name{csv_options}
\alias{csv_options}
\title{Parse comma-separated chunk options}
\usage{
csv_options(x)
}
\arguments{
\item{x}{The chunk options as a string.}
}
\value{
A list of chunk options.
}
\description{
For \pkg{knitr} and R Markdown documents, code chunk options can be written
using the comma-separated syntax (e.g., \verb{opt1=value1, opt2=value2}). This
function parses these options and returns a list. If an option is not named,
it will be treated as the chunk label.
}
\examples{
xfun::csv_options("foo, eval=TRUE, fig.width=5, echo=if (TRUE) FALSE")
}
